/**
 * See license.txt
 */

/******************************************************************************
 * Ping Constants
 *****************************************************************************/
const RUN_INSTALL = Ci.ffIPingService.RUN_INSTALL;
const RUN_UPGRADE = Ci.ffIPingService.RUN_UPGRADE;
const RUN_UNINSTALL = Ci.ffIPingService.RUN_UNINSTALL;

/******************************************************************************
 * Interfaces used monitor upgrade, install, uninstall process and
 * notify the server when changes occur.
 * 
 * @status    FROZEN
 * @version   1.0
 ******************************************************************************/
function PingService() 
{
  //setup additional interfaces
  this._ifaces.push(Ci.nsIObserver);
  this._ifaces.push(Ci.nsISupportsWeakReference);
  this._ifaces.push(Ci.nsIRDFObserver);
  
  //setup a new error
  this._error = Cc["@ensolis.com/forecastfox/error-item;1"].
                createInstance(Ci.ffIErrorItem);        
}
PingService.prototype = {
  __proto__: new ServiceBase("PingService"),
  _install: null,
  _obSvc: null,
  _dtaSrc: null,
  
  ////////////////////////////
  // nsIObserver 
  observe: function PingService_observe(aSubject, aTopic, aData) 
  {
    //only do something if item is us
    var item = aSubject.QueryInterface(Ci.nsIUpdateItem);
    if (item.id != "{0538E3E3-7E9B-4d49-8831-A227C80A7AD3}")
      return;
    
    //get the migrator service
    var mgrSvc = Cc["@ensolis.com/forecastfox/manager-service;1"].
                 getService(Ci.ffIManagerService);
    var migSvc = mgrSvc.migrator;
        
    //determine action based on message  
    switch (aData) {
        
    //being upgraded
    case "item-upgraded":
        
      //going to version less than tracked
      if (migSvc.compare(item.version, "0.7") < 0)
        this._send(RUN_UNINSTALL, "0.9.5", "*");
      break;
          
    //being uninstalled
    case "item-uninstalled":
      this._send(RUN_UNINSTALL, "0.9.5", "*"); 
      break;
        
    //being cancelled
    case "item-cancel-action":
      this.run();
      break;
    }
  },
   
  ///////////////////////////
  // nsIRDFObserver
  onAssert: function PingService_onAssert(aDS, aSource, aProperty, aTarget) 
  { 
    this._dsObserve(aDS, aSource, aProperty, aTarget); 
  },
  onChange: function PingService_onChange(aDS, aSource, aProperty, aOldTarget, aNewTarget) 
  {
    this._dsObserve(aDS, aSource, aProperty, aNewTarget); 
  },  
  onUnassert: function PingService_onUnassert(aDS, aSource, aProperty, aTarget) {},
  onMove: function PingService_onMove(aDS, aOldSource, aNewSource, aProperty, aTarget) {},
  onBeginUpdateBatch: function PingService_onBeginUpdateBatch(aDS) {},
  onEndUpdateBatch: function PingService_onEndUpdateBatch(aDS) {}, 
                   
  ////////////////////////////////
  // ffIService  
  
  /**
   * Initialize the component.  Called by the manager service.
   */  
  start: function PingService_start()
  {
    //set install flag
    this._install = false;
                    
    //get extension manager - toolkit 1.0
    if ("@mozilla.org/extensions/manager;1" in Cc) {
      var exMgr = Cc["@mozilla.org/extensions/manager;1"].
                  getService(Ci.nsIExtensionManager);
             
      //manager supports observers - toolkit 1.5 or greater
      if ("getItemForID" in exMgr) {
        this._obSvc = Cc["@mozilla.org/observer-service;1"].
                    getService(Ci.nsIObserverService);        
        this._obSvc.addObserver(this, "em-action-requested", false);
      
      //manager doesn't support observers - toolkit 1.0
      } else {
          this._dtaSrc = exMgr.datasource;
          this._dtaSrc.AddObserver(this);
      }
    }
    
    //return success
    return true;
  },
  
  /**
   * Destroy the component.  Called by the manager service.  This may be
   * called prior to start so it needs to be safe.
   */  
  stop: function PingService_stop()
  {
    //unregister from the observer service
    if (this._obSvc) 
      this._obSvc.removeObserver(this, "em-action-requested");
      
    //unregister as an observer of the em datasource  
    if (this._dtaSrc)
      this._dtaSrc.RemoveObserver(this);
    
    //remove variables
    this._install = null;
    this._obSvc = null;
    this._dtaSrc = null;
  },
                
  ////////////////////////////////
  // ffIPingService  
    
  /**
   * Run ping command.
   */  
  run: function PingService_run()
  {
    //get preferences
    var pinged = getPref("pinged");
    var migrated = getPref("migrated");
          
    //use migrated value if pinged is blank
    if (pinged == "" && migrated != "")
      pinged = migrated;
    
    //determine ping command and send
    switch (pinged) {
    
      //same version do nothing
      case "0.9.5":
        break;
        
      //never installed or previously uninstalled.    
      case "":
      case "*":
        this._send(RUN_INSTALL, "", "0.9.5");
        break;
        
      //upgrading from old version
      default:       
        this._send(RUN_UPGRADE, pinged, "0.9.5");
        break;
    }
  },
  
  ////////////////////////////
  // Internal Functions 
  
  /**
   * Build query string and send request.
   *
   * @param   Type of ping command to perform.
   * @param   Version we are coming from.
   * @param   Version we are going to.
   */    
  _send: function PingService__send(aType, aFrom, aTo)
  {
    //create query url
    var query = "http://forecastfox.ensolis.com/logs/log.php"; 
    var guid = getGUID();
    query += "?type=" + aType;
    query += "&from=" + aFrom;
    query += "&to=";
    query += (aTo == "*") ? "" : aTo;
    query += "&guid=" + guid;
    
    //send ping request    
    
    var request = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].
                  createInstance(Ci.nsIXMLHttpRequest);      
    request.open("GET", query, true);
    
    //update channel priority - toolkit 1.5 or greater
    if ("nsISupportsPriority" in Ci) {
      if (request.channel instanceof Ci.nsISupportsPriority)
        request.channel.priority = Ci.nsISupportsPriority.PRIORITY_HIGHEST;
    }
    request.send(null);  
    
    //set pref as to version
    setPref("pinged", aTo);
    
    //log the ping in the errors.log for help debugging.  
    var mgrSvc = Cc["@ensolis.com/forecastfox/manager-service;1"].
                 getService(Ci.ffIManagerService);
    var dskSvc = mgrSvc.disk;
    switch (aType) {
    case RUN_INSTALL:
      dskSvc.log("Installing version " + aTo + " on " + guid, null, null);
      break;
    case RUN_UNINSTALL:
      dskSvc.log("Uninstalling version " + aFrom + " on " + guid, null, null);
      break;
    case RUN_UPGRADE:
      dskSvc.log("Upgrading version " + aFrom + " to " + aTo + " on " + guid, 
                 null, null);
      break;
    }
  },
  
  /**
   * Datasource observer function.  Determines the action to run
   * based on datasource updates.
   *
   * @param   The Datasource being updated.
   * @param   The subject being updated.
   * @param   The predicate being updated.
   * @param   The object being updated.
   */
  _dsObserve: function PingService__dsObserve(aDS, aSource, aProperty, aTarget)
  {
    //only do something if item is us      
    if (aSource.ValueUTF8 != "urn:mozilla:extension:" + "{0538E3E3-7E9B-4d49-8831-A227C80A7AD3}")
      return;
    
    //get the migrator service
    var mgrSvc = Cc["@ensolis.com/forecastfox/manager-service;1"].
                 getService(Ci.ffIManagerService);
    var migSvc = mgrSvc.migrator;
    
    //determine action based on predicate  
    var prefix = "http://www.mozilla.org/2004/em-rdf#";    
    var action = aProperty.ValueUTF8.replace(prefix, "");
    switch (action) {
    
    //being installed
    case "toBeInstalled":
      this._install = true;
      break;
      
    //being upgraded
    case "version":
      if (!this._install)
        break;
        
      //get extension item
      var exMgr = Cc["@mozilla.org/extensions/manager;1"].
                  getService(Ci.nsIExtensionManager);      
      var item = exMgr.getItemList("{0538E3E3-7E9B-4d49-8831-A227C80A7AD3}", Ci.nsIUpdateItem.TYPE_EXTENSION,
                                   {})[0];
        
      //going to version less than tracked
      if (migSvc.compare(item.version, "0.7") < 0)
        this._send(RUN_UNINSTALL, "0.9.5", "*");
      break;
            
    //being uninstalled               
    case "toBeUninstalled":
      this._send(RUN_UNINSTALL, "0.9.5", "*"); 
      break;
      
    default:
      break;
    }
  } 
};